import { __awaiter, __decorate, __rest } from "tslib";
import { ClientSideRowModelModule } from '@ag-grid-community/client-side-row-model';
import { ComponentUtil, Events, Grid, ModuleRegistry } from '@ag-grid-community/core';
import { CsvExportModule } from '@ag-grid-community/csv-export';
import { insertDocumentCSSRule, LifecycleMixin, respondToVisibility, } from '@genesislcap/foundation-utils';
import { attr, DOM, observable } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { classNames } from '@microsoft/fast-web-utilities';
import * as changeCase from 'change-case';
import debounce from 'lodash.debounce';
import { GridProCell } from './cell';
import { SelectEditor, NumberEditor, MultiselectEditor, DateEditor, StringEditor, } from './cell-editors';
import { ActionRenderer, ActionsMenuRenderer, BooleanRenderer, EditableRenderer, } from './cell-renderers';
import { GridProColumn } from './column';
import { GridProClientSideDatasource, GridProServerSideDatasource } from './datasource';
import { agThemeFontFaceMap, defaultAgGridFontFace } from './external';
import { GridProGenesisDatasource, gridProGenesisDatasourceEventNames, } from './grid-pro-genesis-datasource';
import { DEBOUNCED_RESIZE_TIME, gridProEventNames, gridProErrorNames, } from './grid-pro.definitions';
import { foundationGridProStyles as styles } from './grid-pro.styles';
import { gridProTemplate as template } from './grid-pro.template';
import { AgGridTheme, agThemeFontsId, agThemeTokenMapClassname, ERROR_COLUMN_FIELD, EDITED_COLUMN_FIELD, GridOptionsConfig, GridProRendererTypes, GRID_READY_EVENT, } from './grid-pro.types';
import { StatePersistence } from './state-persistence';
import { ErrorTooltip } from './tooltips';
import { convertToKebabCase, logger, mergeAndDedupColDefWithColumnState } from './utils';
GridProGenesisDatasource;
GridProClientSideDatasource;
GridProServerSideDatasource;
GridProCell;
GridProColumn;
ModuleRegistry.registerModules([CsvExportModule, ClientSideRowModelModule]);
const DEBOUNCE_TIME = 400;
/**
 * @remarks List of AG properties without rowData
 * @privateRemarks We're excluding rowData as it's a special case that we handle separately
 * @internal
 */
export const AG_PROPERTIES = ComponentUtil.ALL_PROPERTIES.filter((p) => p !== 'rowData');
/**
 * The Grid Pro element.
 * @remarks Grid Pro is a Web Component wrapper around the AG Grid Community library.
 * @public
 * @tagname %%prefix%%-grid-pro
 */
export class GridPro extends LifecycleMixin(FoundationElement) {
    rowDataChanged(_, rowData) {
        if (this.initialised) {
            this.gridApi.setRowData(rowData);
        }
    }
    themeChanged(oldValue, newValue) {
        if (!newValue) {
            this.classNames = '';
            this.destroyFontFace();
            return;
        }
        this.classNames = classNames(agThemeTokenMapClassname(newValue), newValue);
        this.insertFontFace(newValue);
    }
    getfilterModel() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.persistFilterModelKey) {
                return this._filterConfig;
            }
            const filterModel = yield this.statePersistence.getFilterModel(this.persistFilterModelKey);
            return filterModel;
        });
    }
    setFilterModel(value) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.persistFilterModelKey) {
                this._filterConfig = value;
                return;
            }
            yield this.statePersistence.saveFilterModel(this.persistFilterModelKey, value);
        });
    }
    constructor() {
        super();
        this.gridErrorItems = [];
        // Genesis-specific attrs
        this.autoCellRendererByType = false;
        this.onlyTemplateColDefs = false;
        // AG-specific attrs
        /**
         * Boolean attribute to control whether the grid autosizes the columns upon interaction.
         * This will disable the column widths from being manually set, and doesn't save the widths
         * in local storage if you are using `persist-column-state-key`.
         */
        this.gridAutosizingEnabled = false;
        /**
         * The index to add new rows to when using `applyTransaction` or `applyTransactionAsync`
         */
        this.addIndex = 0;
        /**
         * Whether to use the `applyTransactionAsync` function for *add* transactions
         * @remarks Defaults to false
         */
        this.asyncAdd = false;
        /**
         * Whether to use the `applyTransactionAsync` function for *remove* transactions
         * @remarks Defaults to false
         */
        this.asyncRemove = false;
        /**
         * Whether to use the `applyTransactionAsync` function for *update* transactions
         * @remarks Defaults to true
         */
        this.asyncUpdate = true;
        /**
         * If false, will disable cell flashing for all cells by default, unless otherwise defined in custom colDef
         * @remarks Defaults to true
         */
        this.enableCellFlashing = false;
        /**
         * @deprecated - use `enableRowFlashing` instead.
         */
        this.enabledRowFlashing = false;
        /**
         * If true, will enable row flashing for all rows for `add` async transactions
         */
        this.enableRowFlashing = false;
        this.gridFontFace = defaultAgGridFontFace;
        this.columnComponentName = 'grid-pro-column';
        this.theme = AgGridTheme.alpine;
        this.rehydrationAttempted = false;
        this.rootEventsListeners = [];
        this.gridEventsQueue = [];
        this._filterConfig = undefined;
        this.agAttributes = {};
        this.agPropertiesMap = AG_PROPERTIES.reduce((map, property) => {
            map[property.toLowerCase()] = property;
            map[convertToKebabCase(property)] = property;
            return map;
        }, {});
        const hideDataSourceError = this.hideDatasourceError.bind(this);
        this.rootEventsListeners.push({ key: gridProGenesisDatasourceEventNames.dataInit, action: hideDataSourceError }, { key: gridProEventNames.datasourceErrorClose, action: hideDataSourceError }, {
            key: gridProGenesisDatasourceEventNames.error,
            action: this.handleError.bind(this, gridProErrorNames.datasource),
        });
    }
    /**
     * Adds an event listener to the grid element.
     * @param eventType - The event type to listen for. Recommened to use the `Events` from AG Grid.
     * @param listener - The event listener to call when the event is fired.
     * @param options - An options object that specifies characteristics about the event listener.
     * @privateRemarks
     * This will change once we do the AG Upgrade and `eventType` will be correctly typed.
     */
    addEventListener(eventType, listener, options) {
        super.addEventListener(eventType, listener, options);
        if (ComponentUtil.PUBLIC_EVENTS.includes(eventType)) {
            if (this.gridApi) {
                this.addGridEventListener(eventType);
            }
            else {
                this.gridEventsQueue.push(eventType);
            }
        }
    }
    addGridEventListener(eventType) {
        this.gridApi.addEventListener(eventType, (e) => {
            this.$emit(eventType, e);
        });
    }
    handleError(type, event) {
        this.gridErrorItems = [...this.gridErrorItems, { detail: event.detail, type }];
    }
    hideDatasourceError() {
        this.gridErrorItems = this.gridErrorItems.filter((item) => item.type !== gridProErrorNames.datasource);
    }
    connectedCallback() {
        super.connectedCallback();
        if (this.gridAutosizingEnabled) {
            respondToVisibility(this, (gridVisible) => {
                if (gridVisible) {
                    this.debouncedColumnAutosize.bind(this)();
                }
            });
        }
        this.rehydrationAttempted = false;
        if (!this.shouldRunConnect)
            return;
        AG_PROPERTIES.forEach((key) => {
            Object.defineProperty(this, key, {
                set: (v) => {
                    this.agAttributeChangedCallback(convertToKebabCase(key), undefined, v);
                },
                enumerable: true,
                configurable: true,
            });
        });
        this.rootEventsListeners.forEach(({ key, action }) => {
            this.addEventListener(key, action);
        });
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        this.rehydrationAttempted = false;
        const shouldRunDisconnect = this.shouldRunDisconnect;
        this.rootEventsListeners.forEach(({ key, action }) => {
            this.removeEventListener(key, action);
        });
        DOM.queueUpdate(() => {
            var _a;
            if (!shouldRunDisconnect)
                return;
            this.initialised = false;
            (_a = this.gridApi) === null || _a === void 0 ? void 0 : _a.destroy();
        });
    }
    combineAllGridComponents(gridOptionsComponents) {
        const defaultFoundationAgComponents = {
            [GridProRendererTypes.action]: ActionRenderer,
            [GridProRendererTypes.actionsMenu]: ActionsMenuRenderer,
            [GridProRendererTypes.boolean]: BooleanRenderer,
            [GridProRendererTypes.editable]: EditableRenderer,
            [GridProRendererTypes.selectEditor]: SelectEditor,
            [GridProRendererTypes.numberEditor]: NumberEditor,
            [GridProRendererTypes.multiselectEditor]: MultiselectEditor,
            [GridProRendererTypes.dateEditor]: DateEditor,
            [GridProRendererTypes.stringEditor]: StringEditor,
        };
        return Object.assign(Object.assign(Object.assign({}, defaultFoundationAgComponents), gridOptionsComponents), this.gridComponents);
    }
    statePersistanceEnabled() {
        if (!this.persistColumnStateKey || this.persistColumnStateKey.length === 0) {
            return false;
        }
        return true;
    }
    /**
     * Because we try and resize the grid to fit using callbacks on events, this will cause
     * a stack overflow of calls to resize, so we need to debounce it.
     * @internal
     */
    debouncedColumnAutosize() {
        if (!this.gridAutosizingEnabled)
            return;
        if (this.debounced) {
            clearTimeout(this.debounced);
            this.debounced = null;
        }
        this.debounced = setTimeout(() => {
            var _a;
            (_a = this.gridApi) === null || _a === void 0 ? void 0 : _a.sizeColumnsToFit();
        }, DEBOUNCED_RESIZE_TIME);
    }
    /**
     * If {@link @genesislcap/grid-pro#GridPro.gridAutosizingEnabled} is true, will remove the widths from the column states.
     * @internal
     */
    removeConfigWidthsToAutosize(colState) {
        return colState.map((col) => (Object.assign(Object.assign({}, col), { width: null })));
    }
    saveColumnState() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.rehydrationAttempted && this.statePersistanceEnabled()) {
                let colState = this.columnApi.getColumnState();
                colState = this.gridAutosizingEnabled
                    ? this.removeConfigWidthsToAutosize(colState)
                    : colState;
                if (colState.length === 0 || JSON.stringify(colState) === JSON.stringify(this.columnState))
                    return;
                yield this.statePersistence.saveColumnState(this.persistColumnStateKey, colState);
            }
        });
    }
    /**
     * Gets the saved grid ColumnState[] from storage
     * @remarks This is used to restore the grid state when the grid is rehydrated
     * @privateRemarks Not using `columnApi.get|setColumnState()` for these flows as it doesn't work setting new columnDefs vs different ordering.
     * @public
     */
    getSavedColumnState() {
        return __awaiter(this, void 0, void 0, function* () {
            this.columnState = [];
            if (this.statePersistanceEnabled()) {
                this.columnState = yield this.statePersistence.getColumnState(this.persistColumnStateKey);
            }
            return this.columnState;
        });
    }
    restoreColumnState() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.rehydrationAttempted = true;
            const colState = yield this.getSavedColumnState();
            if (colState && colState.length > 0) {
                (_a = this.columnApi) === null || _a === void 0 ? void 0 : _a.applyColumnState({
                    state: colState,
                    applyOrder: true,
                });
            }
        });
    }
    cacheFilterConfig() {
        var _a;
        this.setFilterModel(((_a = this.gridApi) === null || _a === void 0 ? void 0 : _a.getFilterModel()) || undefined);
    }
    restoreCachedFilterConfig() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const filterModel = yield this.getfilterModel();
            if (typeof filterModel !== 'undefined') {
                (_a = this.gridApi) === null || _a === void 0 ? void 0 : _a.setFilterModel(filterModel);
                (_b = this.gridApi) === null || _b === void 0 ? void 0 : _b.onFilterChanged();
            }
        });
    }
    /**
     * @public
     */
    get gridOptions() {
        return this.agGridOptions;
    }
    /**
     * @public
     */
    get gridParams() {
        return {
            globalEventListener: this.globalEventListener.bind(this),
        };
    }
    /**
     * Get the datasource element.
     * @remarks This will return the first datasource element found in the light DOM. Can be either `grid-pro-genesis-datasource`, `grid-pro-client-side-datasource`, or `grid-pro-server-side-datasource`.
     * @public
     */
    get gridProDatasource() {
        return (this.querySelector('grid-pro-genesis-datasource') ||
            this.querySelector('grid-pro-client-side-datasource') ||
            this.querySelector('grid-pro-server-side-datasource'));
    }
    set gridOptions(options) {
        const debounceSaveColumnState = debounce(this.saveColumnState.bind(this), DEBOUNCE_TIME);
        const debouceSaveFiltermodel = debounce(this.cacheFilterConfig.bind(this), DEBOUNCE_TIME);
        const gridOnChangeCallback = () => {
            debounceSaveColumnState();
            this.debouncedColumnAutosize.bind(this)();
        };
        Array.from(this.attributes).forEach((attribute) => {
            const attrName = this.agPropertiesMap[attribute.name];
            if (!attrName)
                return;
            this.agAttributes[attrName] = attribute.value;
        });
        const { columnDefs, components, defaultColDef, getRowId, onGridReady, onFilterChanged } = options, rest = __rest(options, ["columnDefs", "components", "defaultColDef", "getRowId", "onGridReady", "onFilterChanged"]);
        const derivedOptions = Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, this.gridOptionsConfig), { defaultColDef: Object.assign({ enableCellChangeFlash: this.enableCellFlashing, filter: true, resizable: true, sortable: true }, defaultColDef), components: this.combineAllGridComponents(components), suppressDragLeaveHidesColumns: true }), this.eventsAndCallbacks), { onGridReady: (event) => {
                var _a, _b;
                this.gridApi = (_a = options.api) !== null && _a !== void 0 ? _a : event.api;
                this.columnApi = (_b = options.columnApi) !== null && _b !== void 0 ? _b : event.columnApi;
                if (this.gridEventsQueue.length) {
                    this.gridEventsQueue.forEach((type) => {
                        this.addGridEventListener(type);
                    });
                }
                if (this.rowData) {
                    this.gridApi.setRowData(this.rowData);
                }
                if (onGridReady) {
                    onGridReady(event);
                }
                this.$emit(Events.EVENT_GRID_READY, {
                    event,
                    gridId: this.id,
                });
                // TODO: Remove this event when we do the AG Upgrade
                this.$emit(GRID_READY_EVENT, {
                    event,
                    gridId: this.id,
                });
                this.restoreColumnState.bind(this)();
                this.debouncedColumnAutosize.bind(this)();
            }, onColumnPinned: gridOnChangeCallback, onColumnResized: gridOnChangeCallback, onColumnMoved: gridOnChangeCallback, onDisplayedColumnsChanged: gridOnChangeCallback, onFilterChanged: (filterChangedEvent) => {
                debouceSaveFiltermodel();
                if (onFilterChanged) {
                    onFilterChanged(filterChangedEvent);
                }
            }, onGridSizeChanged: gridOnChangeCallback, onSortChanged: gridOnChangeCallback }), rest);
        if (this.gridProDatasource) {
            derivedOptions.columnDefs = columnDefs;
            derivedOptions.getRowId = getRowId;
            this.setLocalGridOptions(derivedOptions);
        }
        else {
            this.mergeAllColumnDefsAndStates(columnDefs).then((value) => {
                derivedOptions.columnDefs = value;
                derivedOptions.getRowId = getRowId;
                this.setLocalGridOptions(derivedOptions);
                this.initGrid();
            });
        }
    }
    setLocalGridOptions(derivedOptions) {
        this.agGridOptions = Object.assign(Object.assign({}, this.agGridOptions), ComponentUtil.copyAttributesToGridOptions(derivedOptions, this.agAttributes));
    }
    initGrid() {
        DOM.queueUpdate(() => {
            if (this.initialised) {
                // Needed due gridSlot children cleanup
                if (this.gridSlot.firstChild) {
                    // prevent re-instantiating the grid using the same slot content
                    this.gridSlot.removeChild(this.gridSlot.firstChild);
                }
            }
            this.agGrid = new Grid(this.gridSlot, this.agGridOptions, this.gridParams);
            this.initialised = true;
        });
    }
    /**
     * Will merge templated column definitions with `columnDefs` plus localStorage's column state or `deferredColumnStates`.
     * @deprecated use `applyTemplateDefinitions` instead.
     * @param columnDefs - The column definitions to apply to the grid.
     * @param deferredColumnDefsOrState - If true, will merge the column definitions with the saved column state or the `deferredColumnStates` property on the `grid-pro-genesis-datasource` element.
     * @returns The merged column definitions.
     */
    applyTemplateDefinitions(columnDefs, deferredColumnDefsOrState = false) {
        logger.deprecated('applyTemplateDefinitions', 'Use `mergeAllColumnDefsAndStates` instead.');
        return this.mergeAllColumnDefsAndStates(columnDefs, deferredColumnDefsOrState);
    }
    /**
     * Will merge templated column definitions with `columnDefs` plus localStorage's column state or `deferredColumnStates`.
     * @public
     * @remarks This will favor the column state from the browser's local storage over the `deferredColumnStates` property on the `grid-pro-genesis-datasource` element.
     * @param columnDefs - The column definitions to apply to the grid.
     * @param deferredColumnDefsOrState - If true, will merge the column definitions with the saved column state or the `deferredColumnStates` property
     * on the `grid-pro-genesis-datasource` element. Defaults to false but `grid-pro-genesis-datasource` will set this to true.
     * @returns The merged column definitions.
     */
    mergeAllColumnDefsAndStates(columnDefs, deferredColumnDefsOrState = false) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const columnDefinitionsFromMetadata = columnDefs || [];
            columnDefinitionsFromMetadata.forEach((colDefFromMetadata) => {
                if (this.headerCaseType && colDefFromMetadata.field && !colDefFromMetadata.headerName) {
                    const changeCaseByType = changeCase[this.headerCaseType];
                    if (changeCaseByType) {
                        colDefFromMetadata.headerName = changeCaseByType(colDefFromMetadata.field);
                    }
                    else {
                        logger.warn(`Invalid header-case-type attribute value: ${this.headerCaseType}. Please use one of the following: camelCase, capitalCase, dotCase, headerCase, noCase, paramCase, pascalCase, pathCase, sentenceCase, snakeCase.`);
                    }
                }
            });
            const templateColumnDefs = Array.from(this.querySelectorAll(this.columnComponentName))
                .map((templateColumn) => templateColumn.definition)
                .map((columnDef) => {
                // enable cell flashing for each column if unspecified on column level but enabled on grid level
                if (this.enableCellFlashing && columnDef.enableCellChangeFlash === undefined) {
                    columnDef.enableCellChangeFlash = true;
                }
                if (columnDef.editable) {
                    if (!columnDef.cellRenderer && !columnDef.cellRendererSelector) {
                        columnDef.cellRendererSelector = (params) => {
                            if (params.data[EDITED_COLUMN_FIELD] === columnDef.field) {
                                return {
                                    component: GridProRendererTypes.editable,
                                };
                            }
                        };
                    }
                    if (!columnDef.tooltipComponent && !columnDef.tooltipValueGetter) {
                        columnDef.tooltipComponent = ErrorTooltip;
                        columnDef.tooltipValueGetter = (params) => {
                            const error = params.data[ERROR_COLUMN_FIELD];
                            if (error && error.column === columnDef.field) {
                                return error.message;
                            }
                        };
                    }
                    if (!columnDef.cellClassRules) {
                        columnDef.cellClassRules = {
                            error: (params) => { var _a; return ((_a = params.data[ERROR_COLUMN_FIELD]) === null || _a === void 0 ? void 0 : _a.column) === params.colDef.field; },
                        };
                    }
                }
                const metadataColumnDef = columnDefinitionsFromMetadata.find((colDef) => columnDef.field === colDef.field);
                return Object.assign(Object.assign({}, metadataColumnDef), columnDef);
            });
            const templateColumnKeys = templateColumnDefs.map((x) => x.field);
            const colDefsToReturn = [
                ...templateColumnDefs,
                ...(this.onlyTemplateColDefs
                    ? []
                    : columnDefinitionsFromMetadata.filter((g) => !templateColumnKeys.includes(g.field))),
            ];
            if (this.gridProDatasource && deferredColumnDefsOrState) {
                const savedColumnState = yield this.getSavedColumnState();
                if ((savedColumnState === null || savedColumnState === void 0 ? void 0 : savedColumnState.length) > 0) {
                    return mergeAndDedupColDefWithColumnState(colDefsToReturn, savedColumnState);
                }
                else if (((_a = this.gridProDatasource.deferredColumnStates) === null || _a === void 0 ? void 0 : _a.length) > 0) {
                    return mergeAndDedupColDefWithColumnState(colDefsToReturn, this.gridProDatasource.deferredColumnStates);
                }
            }
            return colDefsToReturn;
        });
    }
    get observedAttributes() {
        // allow properties to be supplied either lowercased or kebab-cased
        // this allows the user to either supply (for example) enableSorting or enabled-sorting
        // properties lowercased
        const lowerCasedPropertyNames = AG_PROPERTIES.map((property) => property.toLowerCase());
        // properties kebab-cased
        const kebabCasePropertyNames = AG_PROPERTIES.map((property) => convertToKebabCase(property));
        return lowerCasedPropertyNames.concat(kebabCasePropertyNames);
    }
    agAttributeChangedCallback(attName, oldValue, newValue) {
        if (oldValue === newValue) {
            return;
        }
        const gridPropertyName = this.agPropertiesMap[attName];
        if (gridPropertyName) {
            // for properties set before gridOptions is called
            this.agAttributes[gridPropertyName] = newValue;
            if (this.initialised) {
                // for subsequent (post gridOptions) changes
                const changeObject = {};
                changeObject[gridPropertyName] = { currentValue: newValue };
                ComponentUtil.processOnChange(changeObject, this.gridApi);
            }
        }
    }
    globalEventListener(eventType, event) {
        const eventLowerCase = eventType.toLowerCase();
        const browserEvent = new Event(eventLowerCase);
        const browserEventNoType = browserEvent;
        browserEventNoType.agGridDetails = event;
        // for when defining events via myGrid.addEventListener('columnresized', function (event) {...
        this.dispatchEvent(browserEvent);
        // for when defining events via myGrid.oncolumnresized = function (event) {....
        const callbackMethod = 'on' + eventLowerCase;
        if (typeof this[callbackMethod] === 'function') {
            this[callbackMethod](browserEvent);
        }
    }
    insertFontFace(newValue) {
        const fontFaceRule = agThemeFontFaceMap[newValue] || this.gridFontFace;
        this.destroyFontFace();
        const newDestroy = insertDocumentCSSRule(fontFaceRule, agThemeFontsId(newValue));
        if (newDestroy) {
            this.fontFaceDestroy = newDestroy;
        }
    }
    destroyFontFace() {
        if (this.fontFaceDestroy) {
            this.fontFaceDestroy();
            this.fontFaceDestroy = null;
        }
    }
}
__decorate([
    observable
], GridPro.prototype, "columnApi", void 0);
__decorate([
    observable
], GridPro.prototype, "gridApi", void 0);
__decorate([
    observable
], GridPro.prototype, "gridErrorItems", void 0);
__decorate([
    StatePersistence
], GridPro.prototype, "statePersistence", void 0);
__decorate([
    attr({ attribute: 'auto-cell-renderer-by-type', mode: 'boolean' })
], GridPro.prototype, "autoCellRendererByType", void 0);
__decorate([
    attr({ attribute: 'only-template-col-defs', mode: 'boolean' })
], GridPro.prototype, "onlyTemplateColDefs", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'grid-autosizing' })
], GridPro.prototype, "gridAutosizingEnabled", void 0);
__decorate([
    attr({ attribute: 'add-index' })
], GridPro.prototype, "addIndex", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'async-add' })
], GridPro.prototype, "asyncAdd", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'async-remove' })
], GridPro.prototype, "asyncRemove", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'async-update' })
], GridPro.prototype, "asyncUpdate", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'enable-cell-flashing' })
], GridPro.prototype, "enableCellFlashing", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'enabled-row-flashing' })
], GridPro.prototype, "enabledRowFlashing", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'enable-row-flashing' })
], GridPro.prototype, "enableRowFlashing", void 0);
__decorate([
    attr({ attribute: 'persist-column-state-key' })
], GridPro.prototype, "persistColumnStateKey", void 0);
__decorate([
    attr({ attribute: 'persist-filter-model-key' })
], GridPro.prototype, "persistFilterModelKey", void 0);
__decorate([
    attr({ attribute: 'header-case-type' })
], GridPro.prototype, "headerCaseType", void 0);
__decorate([
    observable
], GridPro.prototype, "rowData", void 0);
__decorate([
    observable
], GridPro.prototype, "gridFontFace", void 0);
__decorate([
    attr
], GridPro.prototype, "columnComponentName", void 0);
__decorate([
    observable
], GridPro.prototype, "eventsAndCallbacks", void 0);
__decorate([
    attr
], GridPro.prototype, "headerHeight", void 0);
__decorate([
    attr
], GridPro.prototype, "rowHeight", void 0);
__decorate([
    attr
], GridPro.prototype, "theme", void 0);
__decorate([
    observable
], GridPro.prototype, "classNames", void 0);
__decorate([
    observable
], GridPro.prototype, "agGrid", void 0);
__decorate([
    observable
], GridPro.prototype, "gridSlot", void 0);
__decorate([
    observable
], GridPro.prototype, "gridComponents", void 0);
__decorate([
    GridOptionsConfig
], GridPro.prototype, "gridOptionsConfig", void 0);
/**
 * GridPro shadow root options.
 * @public
 * @remarks
 * For extensibility, this can be used to set the shadow root mode and delegatesFocus
 */
export const foundationGridProShadowOptions = undefined;
/**
 * Grid Pro default configuration.
 * @public
 */
export const defaultGridProConfig = {
    shadowOptions: undefined,
};
/**
 * A function that returns a Foundation Grid Pro registration for configuring the component with a DesignSystem.
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-grid-pro\>
 */
export const foundationGridPro = GridPro.compose(Object.assign({ baseName: 'grid-pro', styles,
    template, shadowOptions: foundationGridProShadowOptions }, defaultGridProConfig));
